Package: globals
Version: 0.19.1
Depends: R (>= 3.1.2)
Imports: codetools
Title: Identify Global Objects in R Expressions
Authors@R: c(
    person("Henrik", "Bengtsson", role=c("aut", "cre", "cph"),
           email="henrikb@braju.com"),
    person("Davis","Vaughan", role="ctb",
           email="davis@posit.co"))
Description: Identifies global ("unknown" or "free") objects in R expressions
    by code inspection using various strategies (ordered, liberal,
    conservative, or deep-first search). The objective of this package is to
    make it as simple as possible to identify global objects for the purpose
    of exporting them in parallel, distributed compute environments.
License: LGPL (>= 2.1)
LazyLoad: TRUE
ByteCompile: TRUE
Language: en-US
Encoding: UTF-8
URL: https://globals.futureverse.org,
        https://github.com/futureverse/globals
BugReports: https://github.com/futureverse/globals/issues
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-03-12 23:06:06 UTC; hb
Author: Henrik Bengtsson [aut, cre, cph],
  Davis Vaughan [ctb]
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Repository: CRAN
Date/Publication: 2026-03-13 07:30:09 UTC
Built: R 4.6.0; ; 2026-04-27 05:37:07 UTC; unix
