# Generated by roxygen2: do not edit by hand

S3method("$<-",Globals)
S3method("[",Globals)
S3method("[<-",Globals)
S3method("[[<-",Globals)
S3method("names<-",Globals)
S3method(as.Globals,Globals)
S3method(as.Globals,default)
S3method(as.Globals,list)
S3method(c,Globals)
S3method(cleanup,Globals)
S3method(packagesOf,Globals)
S3method(unique,Globals)
export(Globals)
export(as.Globals)
export(cleanup)
export(findGlobals)
export(globalsByName)
export(globalsOf)
export(packagesOf)
export(walkAST)
importFrom(codetools,findLocalsList)
importFrom(codetools,makeUsageCollector)
importFrom(codetools,walkCode)
importFrom(utils,capture.output)
importFrom(utils,packageDescription)
importFrom(utils,str)
