# globals: Identify Global Objects in R Expressions


## Installation
R package globals is available on [CRAN](http://cran.r-project.org/package=globals) and can be installed in R as:
```r
install.packages('globals')
```

### Pre-release version

To install the pre-release version that is available in branch `develop`, use:
```r
source('http://callr.org/install#HenrikBengtsson/globals@develop')
```
This will install the package from source.  



## Software status

| Resource:     | CRAN        | Travis CI      | Appveyor         |
| ------------- | ------------------- | -------------- | ---------------- |
| _Platforms:_  | _Multiple_          | _Linux & OS X_ | _Windows_        |
| R CMD check   | <a href="http://cran.r-project.org/web/checks/check_results_globals.html"><img border="0" src="http://www.r-pkg.org/badges/version/globals" alt="CRAN version"></a> | <a href="https://travis-ci.org/HenrikBengtsson/globals"><img src="https://travis-ci.org/HenrikBengtsson/globals.svg" alt="Build status"></a>  | <a href="https://ci.appveyor.com/project/HenrikBengtsson/globals"><img src="https://ci.appveyor.com/api/projects/status/github/HenrikBengtsson/globals?svg=true" alt="Build status"></a> |
| Test coverage |                     | <a href="https://codecov.io/gh/HenrikBengtsson/globals"><img src="https://codecov.io/gh/HenrikBengtsson/globals/branch/develop/graph/badge.svg" alt="Coverage Status"/></a>    |                  |
