% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/walkAST.R
\name{walkAST}
\alias{walkAST}
\title{Walk the Abstract Syntax Tree (AST) of an R Expression}
\usage{
walkAST(
  expr,
  atomic = NULL,
  name = NULL,
  call = NULL,
  pairlist = NULL,
  substitute = FALSE
)
}
\arguments{
\item{expr}{R \link[base]{expression}.}

\item{atomic, name, call, pairlist}{single-argument function that takes an
atomic, name, call and pairlist expression, respectively. Must
return a valid R expression.}

\item{substitute}{If TRUE, \code{expr} is
\code{\link[base]{substitute}()}:ed.}
}
\value{
R \link[base]{expression}.
}
\description{
Walk the Abstract Syntax Tree (AST) of an R Expression
}
\keyword{internal}
\keyword{programming}
