% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cleanup.R
\name{cleanup.Globals}
\alias{cleanup.Globals}
\alias{cleanup}
\title{Drop certain types of globals}
\usage{
\method{cleanup}{Globals}(globals, drop = c("missing", "base-packages", "nativesymbolinfo"), ...)
}
\arguments{
\item{globals}{A Globals object.}

\item{drop}{A character vector specifying what type of globals to drop.}

\item{\ldots}{Not used}
}
\description{
Drop certain types of globals
}
