% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Globals-class.R
\name{Globals}
\alias{Globals}
\alias{as.Globals}
\alias{as.Globals.Globals}
\alias{as.Globals.list}
\alias{[.Globals}
\alias{names}
\title{A representation of a set of globals}
\usage{
Globals(object, ...)
}
\arguments{
\item{object}{A named list.}

\item{\ldots}{Not used.}
}
\value{
An object of class \code{Globals}, which is a \emph{named} list
of the value of the globals, where the element names are the names of
the globals.  Attribute \code{where} is a named list of the same length
and with the same names.
}
\description{
A representation of a set of globals
}
\seealso{
The \code{\link{globalsOf}()} function identifies globals
from an R expression and returns a Globals object.
}
