% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packagesOf.R
\name{packagesOf.Globals}
\alias{packagesOf.Globals}
\alias{packagesOf}
\title{Identify the packages of the globals}
\usage{
\method{packagesOf}{Globals}(globals, ...)
}
\arguments{
\item{globals}{A Globals object.}

\item{\ldots}{Not used.}
}
\value{
Returns a character vector of package names.
}
\description{
Identify the packages of the globals
}
